import logging
import argparse
import uuid
import threading
import sys

import bottle
from knilb import agent

log = logging.getLogger(__name__)

app = bottle.Bottle()
passkey = []


@app.get('/')
def index():
    """Show all routes on web app.

    Returns:
        A dict mapping each URI with available HTTP Method, e.g. {'/': 'GET'}
    """
    return {r.rule: r.method for r in app.routes}

# def _thread_bottle(port, debug=False):
#     log.debug('Starting web server.')
#     for r in app.routes:
#         log.info('{} http://host:{}{}'.format(r.method, port, r.rule))
#     bottle.run(app=app, host='0.0.0.0', port=port, debug=False, reloader=False)


def _thread_agent(user, passkey, cloud):
    try:
        while True:
            a = agent.Agent(user, passkey, cloud)
            if a.register():
                a.run()
            else:
                log.info('Agent register FAIL, try again.')
    except KeyboardInterrupt as e:
        log.debug('Goodbye')
    except Exception as e:
        log.critical(e)
    finally:
        log.debug('All Done')


def main():
    logging.basicConfig(format='%(levelname)s:%(message)s', level=logging.INFO)
    d = '''Pimo contains examples of Device Under Test (using a Pimoroni
           AutomationHAT) and Automated Test Equipment (using a Pimoroni
           ExplorerHAT).'''
    parser = argparse.ArgumentParser(description=d)
    parser.add_argument('app', type=str, choices=['ate', 'dut'],
                        help='''ate for Automated Test Equipment or dut for 
                                device under test.''')
    # group = parser.add_mutually_exclusive_group(required=True)
    # group.add_argument('-a', '--ahat', action='store_true',
    #                    help='Device Under Test using Pimoroni AutomationHAT.')
    # group.add_argument('-e', '--ehat', action='store_true',
    #                    help='''Automated Test Equipment using Pimoroni
    #                         ExplorerHAT Pro.''')
    parser.add_argument('-v', '--verbose', action='store_true',
                        help='Print copious debug statements.')
    parser.add_argument('-p', dest='port', type=int,
                        metavar='<port>', default=8080,
                        help='''Port on localhost of Web App API. Default 8080.
                                Recommended 8081 for ATE or
                                Recommended 8082 for Device Under Test.''')
    parser.add_argument('-c', dest='cloud', type=str, nargs=2,
                        # metavar='<foo> <API Key>', nargs=2,
                        help='''Cloud URL and API key.
                                This causes Agent to run automatically.
                                URL must be <scheme://ipaddress:port>,
                                for example http://192.168.1.2:8080''')
    args = parser.parse_args()

    # Modify logging for verbose
    if args.verbose:
        logging.getLogger().setLevel(logging.DEBUG)
        log.debug('OK, VERBOSE DEBUGGING ON.')

    # DESIGN: automationhat and explorerhat compete for GPIO pins.
    # DESIGN: do conditional import to avoid race condition for pins.
    try:
        if args.app == 'ate':
            from pimoscope import ehat
            subapp = ehat.initialize()
            # passkey = ehat.promt_passkey() if args.cloud else None
            app.mount('ate', subapp)
            log.info('Starting Automation on port {}'.format(args.port))
        if args.app == 'dut':
            from pimoscope import ahat
            subapp = ahat.initialize()
            app.mount('dut', subapp)
            log.info('Starting Explorer on port {}'.format(args.port))
    except RuntimeError as e:
        log.critical(e)
        sys.exit()
    except Exception as e:
        log.critical(e)
        sys.exit()
    else:
        if args.cloud and args.app == 'ate':
            a = threading.Thread(name='Agent={}'.format(uuid.getnode()),
                                 target=_thread_agent,
                                 args=(uuid.getnode(), passkey, args.cloud))
            a.setDaemon(True)
            a.start()
        elif args.cloud:
            log.warning('Agent runs IF ONLY IF configured as ate.')

    for r in app.routes:
        log.info('{} http://host:{}{}'.format(r.method, args.port, r.rule))
    bottle.run(app=app, host='0.0.0.0', port=args.port, debug=False)


if __name__ == '__main__':
    main()



"""Automation HAT web app.

"""
import logging
import os
import sys
from bottle import Bottle, request

if 'raspberrypi' in os.uname():
    import automationhat
else:
    try:
        import mock
    except ImportError:
        import unittest.mock as mock
    finally:    
        sys.modules['explorerhat'] = mock.Mock()
    
log = logging.getLogger(__name__)

app = Bottle()


@app.get('/')
def index():
    """Affirms AutomationHAT is physically present.

    **Example request**:

    .. sourcecode:: http

      GET /dut HTTP/1.1
      Host: 192.168.1.87:8080
      Accept: */*

    **Example response**:

    .. sourcecode:: http

      HTTP/1.0 200 OK
      Server: WSGIServer/0.1 Python/2.7.9
      Content-Length: 25
      Content-Type: application/json

      {"isautomationhat": true}

    :status 200: successful
    :resjson string isautomationhat: True if AutomationHAT is present

    """
    return {'isautomationhat': automationhat.is_automation_hat()}


@app.get('/relay')
def status():
    """Status of all three relays as reported by AutomationHAT software.

    **Example request**:

    .. sourcecode:: http

      GET /dut/relay HTTP/1.1
      Host: 192.168.1.88:8080
      Accept: */*

    **Example response**:

    .. sourcecode:: http

      HTTP/1.0 200 OK
      Server: WSGIServer/0.1 Python/2.7.9
      Content-Length: 44
      Content-Type: application/json

      {"three": false, "two": false, "one": false}

    :status 200: succesful
    :resjson boolean three: application state of relay three
    :resjson boolean two: application state of relay two
    :resjson boolean one: application state of relay one

    """
    return automationhat.relay.is_on()


@app.post('/relay')
def toggle():
    """Toggle of all three relays on AutomationHAT.

    **Example request**:

    .. sourcecode:: http

      POST /dut/relay HTTP/1.1
      Host: 192.168.1.88:8080
      Accept: */*

    **Example response**:

    .. sourcecode:: http

      HTTP/1.0 200 OK
      Server: WSGIServer/0.1 Python/2.7.9
      Content-Length: 41
      Content-Type: application/json

      {"three": true, "two": true, "one": true}

    :status 200: succesful
    :resjson boolean three: application state of relay three
    :resjson boolean two: application state of relay two
    :resjson boolean one: application state of relay one

    """
    automationhat.relay.toggle()
    return automationhat.relay.is_on()


@app.get('/relay/<relay>')
def read(relay):
    """Status of specified relay as reported by AutomationHAT software.

    **Example request**:

    .. sourcecode:: http

      GET /dut/relay/one HTTP/1.1
      Host: 192.168.1.88:8080
      Accept: */*

    **Example response**:

    .. sourcecode:: http

      HTTP/1.0 200 OK
      Server: WSGIServer/0.1 Python/2.7.9
      Content-Length: 13
      Content-Type: application/json

      {"relay": "one", "state": true}

    :status 200: succesful
    :resjson boolean relay: relay identifier, e.g. one, two, three
    :resjson boolean state: application state of relay

    """
    if 'one' in relay:
        r = 0
    elif 'two' in relay:
        r = 1
    elif 'three' in relay:
        r = 2
    return {'relay': relay, 'state': automationhat.relay[r].is_on()}


@app.post('/relay/<relay>')
def control(relay):
    """Control the state of an individual relay.

    **Example request**:

    .. sourcecode:: http

      POST /dut/relay/one HTTP/1.1
      Host: 192.168.1.88:8080
      Accept: */*
      Content-Type: application/json
      Content-Length: 12

    **Example response**:

    .. sourcecode:: http

      HTTP/1.0 200 OK
      Server: WSGIServer/0.1 Python/2.7.9
      Content-Length: 14
      Content-Type: application/json

      {"relay": "one", "state": true}

    :status 200: succesful
    :reqjson int state: 1 to turn ON, 0 to turn OFF
    :resjson boolean relay: relay identifier, e.g. one, two, three
    :resjson boolean state: application state of relay

    """
    if 'one' in relay:
        r = 0
    elif 'two' in relay:
        r = 1
    elif 'three' in relay:
        r = 2
    automationhat.relay[r].write(int(request.json['state']))
    return {'relay': relay, 'state': automationhat.relay[r].is_on()}


def initialize():
    log.debug('Detected AutomationHAT.')
    if automationhat.is_automation_hat():
        automationhat.relay.off()
        automationhat.light.off()
        automationhat.light.power.on()
        automationhat.light.comms.on()
    else:
        raise RuntimeError('Failed to detect AutomationHAT.')
    return app



from setuptools import setup, find_packages

with open('README.md') as f:
    long_description_from_readme = f.read()
    
setup(name='pimoscope',
      version='0.1',
      author='nko',
      author_email='nate@knilb.com',
      description='Test equipment for Knilb.',
      long_description=long_description_from_readme,
      long_description_content_type="text/markdown",
      url='http://knilb.com',
      packages=find_packages(exclude=["deprecated"]),
      classifiers=['License :: OSI Approved :: Apache Software License',
                   'Programming Language :: Python :: 2.7',
                   'Programming Language :: Python :: 3.8'],
      install_requires=['knilb', 'bottle', 'automationhat', 'explorerhat'],
      entry_points={'console_scripts': ['pimo=pimoscope.__main__:main']},
      include_package_data=True,
      zip_safe=False,
      license='Apache')



"""Explorer HAT web app.

"""
import logging
import os
import sys
import time
from bottle import Bottle

if 'raspberrypi' in os.uname():
    import explorerhat
else:
    try:
        import mock
    except ImportError:
        import unittest.mock as mock
    finally:    
        sys.modules['explorerhat'] = mock.Mock()

log = logging.getLogger(__name__)

app = Bottle()
passkey = []


@app.get('/')
def index():
    """Affirms ExplorerHAT is physically present.

    **Example request**:

    .. sourcecode:: http

      GET /ate HTTP/1.1
      Host: 192.168.1.87:8080
      Accept: */*

    **Example response**:

    .. sourcecode:: http

      HTTP/1.0 200 OK
      Server: WSGIServer/0.1 Python/2.7.9
      Content-Length: 23
      Content-Type: application/json

      {"isexplorerpro": true}

    :status 200: successful
    :resjson string isexplorerpro: True if Explorer Pro HAT is present

    """
    return {'isexplorerpro': explorerhat.is_explorer_pro()}


@app.get('/input')
def status():
    """Status of all digital input on ExplorerHAT.

    **Example request**:

    .. sourcecode:: http

      GET /ate/input HTTP/1.1
      Host: 192.168.1.87:8080
      Accept: */*

    **Example response**:

    .. sourcecode:: http

      HTTP/1.0 200 OK
      Server: WSGIServer/0.1 Python/2.7.9
      Content-Length: 43
      Content-Type: application/json

      {"four": 0, "three": 0, "two": 0, "one": 0}

    :status 200: successful
    :resjson int four: status of relay four
    :resjson int three: status of relay three
    :resjson int two: status of relay two
    :resjson int one: status of relay one

    """
    return explorerhat.input.read()


def _handler_passkey(channel, event):
    global passkey
    if event == 'press' and channel <= 4:
        passkey.append(channel)
        explorerhat.light[len(passkey)-1].on()
        log.debug(len(passkey)*'*')


def promt_passkey():
    # get passkey from ExplorerHAT
    global passkey
    passkey = []
    explorerhat.light.off()
    explorerhat.light.pulse()
    while len(passkey) < 4:
        explorerhat.touch.pressed(_handler_passkey)
        time.sleep(0.05)
    log.debug('Passkey is {}'.format(''.join(str(p) for p in passkey)))
    return ''.join(str(p) for p in passkey)


def initialize():
    if explorerhat.is_explorer_pro():
            log.debug('Detected ExplorerHAT Pro.')
            explorerhat.light.off()
            explorerhat.light.green.on()
            explorerhat.light.blue.on()
    else:
        raise RuntimeError('Failed to detect ExplorerHAT Pro.')
    return app
